<?php
/*------------------------------------------------------------------------
# scancircle.php - ScanCircle plugin for Joomla
# ------------------------------------------------------------------------
# author    aklaren
# copyright Copyright (C) 2014-2024 ScanCircle.com. All Rights Reserved.
# @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites: https://www.scancircle.com
# Technical Support:  Forum - https://www.scancircle.com/forum
-------------------------------------------------------------------------*/
	defined('_JEXEC') or die('Restricted access');
	class plgContentScanCircle extends JPlugin {
		const SCANCIRCLE = '/\[scancircle .*?\]/i';
		const RESULTS = '/\[scancircle_results .*?\]/i';
		const PARAMS = '/\s+(\w+)="(.*?)"/';
		
		function onContentPrepare($context, &$article, &$params, $page = 0) {
			$found = false;
			while(preg_match(self::SCANCIRCLE, $article->text, $shortcode)) {
				if(!$found) {
					// *** IMPORTANT: always keep this version number in sync with the one in the XML ***
					$version = '2.9';
					$options = '?plugin=Joomla'.$version;
					$partner = '';
					$language = 'en';
					$secure = 's';
					$env = 'com';
					$jquery = false;
					$expanded = '';
					if(preg_match_all(self::PARAMS, $shortcode[0], $matches, PREG_SET_ORDER)) {
						foreach($matches as $match) {
							$key = $match[1];
							$val = $match[2];
							switch(strtolower($key)) {
								case 'partner':
									if(preg_match('/[^a-z0-9]/i', $val)) {
										$expanded = $this->scancircle_error('Only alphanumerical characters supported in partner parameter');
									} else {
										$partner = strtolower($val);
									}
									break;
								case 'language':
									if(preg_match('/[^a-z]/i', $val)) {
										$expanded = $this->scancircle_error('Only alpha characters supported in language parameter');
									} else {
										$language = strtolower($val);
									}
									break;
								case 'https':
									if($val === '0') { $secure = ''; }
									break;
								case 'env':
									if(strlen($val) > 0 && $val[0] == '/') { $val = $env.$val; }
									if(!preg_match('`^(com|eu)(\/[a-z0-9]+)?$`', $val)) {
										$expanded = $this->scancircle_error('Env parameter should be in the format [TLD][/ENV]');
									} else {
										$env = $val;
									}
									break;
								case 'speedometer':
									$jquery = true;
									// do not break
								default:
									if(preg_match('/[^a-z]/i', $key)) {
										$expanded = $this->scancircle_error('Invalid characters detected in attribute name: '.$key);
									} elseif(preg_match('/["<]/', $val)) {
										$expanded = $this->scancircle_error('Invalid characters detected in attribute: '.$key);
									} else {
										$options .= '&'.$key.'='.rawurlencode($val);
									}
							}
							// exit foreach loop on first error
							if($expanded) { break; }
						}
						if($expanded) {
							// error message already set
						} elseif($partner) {
							$found = true;

							// Joomla strips <script> tags from content, so add to header
							$script = "http$secure://$partner.scancircle.$env/$language/scancircle.js$options";
							if($jquery) {
								JHtml::_('jquery.framework');	// load jQuery if Speedometer is to be used
							}
							$document = JFactory::getDocument();
							$document->addScript($script);

							$expanded = <<< EOL
<div id="scancircle">
	<a href="http://$partner.scancircle.$env/$language/scan" id="scancircle_button" title="ScanCircle Scan">ScanCircle</a>
</div>
EOL;
						} else {
							$expanded = $this->scancircle_error('Syntax: [scancircle partner="PARTNER-CODE" OPTIONS]');
						}
					} else {
						$expanded = $this->scancircle_error('ScanCircle shortcode without parameters');
					}
				} else {
					$expanded = $this->scancircle_error('Only one scan widget allowed per page');
				}
				$article->text = preg_replace(self::SCANCIRCLE, $expanded, $article->text, 1);
			}

			while(preg_match(self::RESULTS, $article->text, $shortcode)) {
				$partner = '';
				$param = '';
				$attribs = '';
				$expanded = '';
				if(preg_match_all(self::PARAMS, $shortcode[0], $matches, PREG_SET_ORDER)) {
					foreach($matches as $match) {
						$key = $match[1];
						$val = $match[2];
						switch(strtolower($key)) {
							case 'partner':
								if(preg_match('/[^a-z0-9]/i', $val)) {
									$expanded = $this->scancircle_error('Only alphanumerical characters supported in partner parameter');
								} else {
									$partner = strtolower($val);
								}
								break;
							case 'param':
								if(preg_match('/[^a-z]/i', $val)) {
									$expanded = $this->scancircle_error('Only alpha characters supported in param parameter');
								} else {
									$param = strtolower($val);
								}
								break;
							case 'width':
							case 'height':
							case 'name':
							case 'loading':
							case 'id':
							case 'class':
							case 'style':
							case 'tabindex':
							case 'title':
							case 'draggable':
								if(preg_match('/["<]/', $val)) {
									$expanded = $this->scancircle_error('Invalid characters detected in parameter: '.$key);
								} else {
									$attribs .= ' '.strtolower($key).'="'. htmlentities($val).'"';
								}
								break;
							default:
								$expanded = $this->scancircle_error('Unsupported attribute: '.$key);
						}
						// exit foreach loop on first error
						if($expanded) { break; }
					}
				}
				if($expanded) {
					// error message already set
				} elseif(!$partner || !$param) {
					$expanded = $this->scancircle_error('Syntax: [scancircle_results partner="PARTNER-CODE" param="URL-PARAMETER" IFRAME-ATTRIBUTES]');
				} elseif(!isset($_GET[$param])) {
					$expanded = $this->scancircle_error('Missing URL-parameter: '.$param);
				} elseif(!filter_var($url = $_GET[$param], FILTER_VALIDATE_URL)) {
					$expanded = $this->scancircle_error('Invalid URL: '.htmlentities($url));
				} elseif(!preg_match('/^https?:\/\/'.$partner.'\.scancircle\.(com|eu)\//i', $url)) {
					$expanded = $this->scancircle_error('Only valid for ScanCircle URLs for partner: '.$partner);
				} else {
					$expanded = '<iframe src="'.$url.'"'.$attribs.'></iframe>';
				}
				$article->text = preg_replace(self::RESULTS, $expanded, $article->text, 1);
			}
			
			return true;
		}
		
		function scancircle_error($msg) {
			return '***ERROR ScanCircle shortcode: '.$msg.' ***';
		}
	}
